Previous: [[Message Queue Component]] | Next: [[Styling]]

------

Most of the options customizable in the flash message component will be set through `flashMessage()`. However, you can also set them by reopening the flash message component - these values will become your defaults. Some additional options are exposed to you in the `{{flash-message}}` component:

- [Properties](#properties)
  - [Action](#class-name)
  - [Class name](#class-name)
  - [Content](#content)
  - [Icon class format](#icon-class-format)
  - [Template](#template)
  - [Type](#type)
- [Animation](#animation)
  - [Set visibility](#set-visibility)
  - [Hide](#hide)
  - [Show](#show)


## Properties

For non-static messages, you will generally set `action`, `content`, `duration`, and `type` using the `flashMessage()` method. However, the aforementioned properties can also be set via the `message` property:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{!-- Can set action, content, duration, and type --}}
{{flash-message message=messageProperties}}
```


### Action

Flash message components will always close when the user clicks them. However, you can also specify an action that will run and bubble up when a message is clicked.

Pass an action name, as a string, as the `action` option:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{flash-message action='saveChoice'}}
```

```js
// app/controllers/some-route.js

import Ember from 'ember';

export default Ember.Controller.extend({
  actions: {
    saveChoice: function() {
      // Do stuff when the message is clicked here
    }
  }
});
```


### Class Name

You can customize the base class name of each flash message by extending the `{{flash-message}}` component in your app tree:

```js
// app/components/flash-message.js

import FlashMessageComponent from 'ember-flash-messages/components/flash-message';

export default FlashMessageComponent.extend({
  className: 'message' // String
});
```

The type is automatically prepended to the class name. For example, a message with `type: 'success'` and `className: 'alert'` will have the following class names:

```html
<dl class=".alert .alert-success"></dl>
```


### Content

To pass content to the flash message, pass a string as the `content` option:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{flash-message content='Here is the message!'}}
```

The `content` option accepts HTML:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{flash-message content='Here is a <strong>bold message!</strong>'}}
```

If you prefer, you can use the flash message component as a block component. This also enables you to use Handlebars in your message:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{#flash-message type='error'}}
  <div>
    {{!-- HTML bars --}}
    <span class="icon-{{type}}"></span>
    {{#link-to 'posts.new'}}Add a new post{{/link-to}}
  </div>
{{/flash-message}}
```


### Icon Class Format

The `{{flash-message}}` component has an `iconClass` property that makes it easy to standardize icons in your flash messages. The `iconClass` defaults to `icon-{{type}}` where `{{type}}` is replaced by the current message type (e.g. success, error, etc).

You can customize the format of `iconClass` by overwriting the `iconClassFormat` property:

```js
// app/components/flash-message.js

import FlashMessageComponent from 'ember-flash-messages/components/flash-message';

export default FlashMessageComponent.extend({
  property: 'icon icon_font_for-{{type}}' // String
});
```


### Template

You can easily change the flash message's template by overwriting `app-name/templates/components/flash-message.hbs`:

```hbs
{{!-- app/templates/components/flash-message.hbs --}}

Your custom Handlebars here.
```

You can reference the [original template file here](https://github.com/sir-dunxalot/ember-flash-messages/blob/master/app/templates/components/message-queue.hbs)



### Type

To set a type for the flash message, pass a string as the `type` option:

```hbs
{{!-- app/templates/some-route.hbs --}}

{{flash-message type='info'}}
```

The `type` option will be used for the message and icon element class names. If can be anything you want - you're not limited by the standard success, error, warning, and info classes.


## Animation

The `{{flash-message}}` component has three methods for handling the hiding and showing of messages: `hide()`, `show()`, and `setVisibility()`.


### Set Visibility

If you're adding custom functionality to the `{{flash-message}}` component you **do not** want to call the `hide()` and `show()` method directly. Instead, you want to call `setVisibility()`.

`setVisibility()` accepts a single boolean parameter:

```js
// app/components/flash-message.js

import FlashMessageComponent from 'ember-flash-messages/components/flash-message';

  someCustomMethod: function() {
    this.setVisibility(true); // Show message
    this.setVisibility(false); // Hide message
  }

});
```


### Hide

To customize how the message is hidden, you can override the `hide()` method:

```js
// app/components/flash-message.js

import FlashMessageComponent from 'ember-flash-messages/components/flash-message';

  hide: function() {
    this.$().velocity('slideUp', {
      display: 'table',
      duration: 500
    });
  }

});
```

If you want to use CSS transitions instead of a JS library, you can use `show()` to add a class to the component:

```js
// app/components/flash-message.js

import FlashMessageComponent from 'ember-flash-messages/components/flash-message';

  hide: function() {
    this.$().addClass('removing');

    Em.run.later(this, function() {
      this.removeClass('removing');
    }, this.get('animationDuration') - 10);
  }

});
```

> Warning: You should not call `hide()` directly from a method in the component. Instead, call `this.setVisibility(false);`.


### Show

To customize how the message is shown, you can override the `show()` method:

```js
// app/components/flash-message.js

import FlashMessageComponent from 'ember-flash-messages/components/flash-message';

  hide: function() {
    if (!Modernizr.csstransitions) {
      this.$().velocity('slideDown', {
        display: 'table',
        delay: 100,
        duration: 250
      });
    }
  }

});
```

If you want to use CSS transitions instead of a JS library, you can use `show()` to add a class to the component, as shown above in [hide()](#hide).

> Warning: You should not call `show()` directly from a method in the component. Instead, call `this.setVisibility(true);`.


------

Previous: [[Message Queue Component]] | Next: [[Styling]]
